' LCD106.Bas
'
' Illustrates capabilities of the NetMedia Basicx 2 X 16 LCD with back light.
' Configured for 9600 baud, 8 bits, no parity.
'
'												LCD #106
'   BX24 (term 12) -----------------------------------> RX (term 7)
'                          
' 
' Opens BX24 terminal 12 as SerCom3, 9600, noninverted, 8 bits per byte, no parity. Sets geometry of LCD as 4X20 
' and clears the LCD and writes a byte in decimal and in hex, an integer, a long, a character and a single to the 
' LCD display.
'
' Defines, special characters 0 - 5 as no vertical lines, 1 vertical line, 2 vertical lines etc.  Displays an
' increasing bar and then a decreasing bar on line 3.
'
' Enters an infinite loop winking outputs 7 and 6.
'
' Compile with SerCom3.Bas
'
' copyright, Peter H. Anderson, Baltimore, MD, Nov, '03

Sub Main()
   
   Dim Num_5 as Byte, N as Byte, NN as Byte, J as Byte, Remainder as Byte
   Dim I as Integer, L as Long, F as Single
   Dim Str as String 

   Call Sleep(3.0)
   
   Call SetupComPort(12, 9600, &H08)	' term 12, 9600 baud, inverted
   Str = "?G420"
   Call PutStr_3(Str)		      	' configure LCD as 4 X 20
      
   Str = "?f"				
   Call PutStr_3(Str)			' clear the LCD and home the cursor
   
   Str = "     LCD #106?n"		' note new line
   Call PutStr_3(Str)
   
   Str = "www.phanderson.com?n"		 
   Call PutStr_3(Str)
      
   Str = "?0?1?2?3?4?5?6?7?n"		' display special characters
   Call PutStr_3(Str)
      
   Call Sleep(5.0)			' pause to admire

   Str = "?y1?x00"
   Call PutStr_3(Str)

   Str = "?l?j?l?j?l?y1"		' clear lines 1, 2 and 3 and start at line 1
   Call PutStr_3(Str)			' note use of down cursor
   
   L = 123456
   I = 12345
   F = 23.75

   For N = 0 to 25
      Str = "?x00?y1"			' locate cursor to beginning of line 1	
      Call PutStr_3(Str)
      Call PutB_3(N)			'  display in decimal 
      Str = "?t"
      Call PutStr_3(Str)
      Call PutHexB_3(N)			' display in hex	
      Str = "?t"
      Call PutStr_3(Str)
      Call PutI_3(I+CInt(N))		' display an integer
      Str = "?n"
      Call PutStr_3(Str)
      Call PutL_3(L + CLng(N))		' display a long
      Str = "?t"
      Call PutStr_3(Str)
      Call PutByte_3(ASC("a") + N)	' display a character
      Str = "?t"
      Call PutStr_3(Str)
      Call PutS_3(F + CSng(N))		' display a single   
      Str = "?g"
      Call PutStr_3(Str)		' beep
      Call Sleep(1.0)
      
   Next

   Str ="?D00000000000000000"		' define special characters
   Call PutStr_3(Str)
   Call Sleep(0.2)	
   Str ="?D11010101010101010"		
   Call PutStr_3(Str)
   Call Sleep(0.2)			' delay to allow write to EEPROM
   Str ="?D21818181818181818"
   Call PutStr_3(Str)
   Call Sleep(0.2)
   Str ="?D31c1c1c1c1c1c1c1c"
   Call PutStr_3(Str)
   Call Sleep(0.2)
   Str ="?D41e1e1e1e1e1e1e1e"
   Call PutStr_3(Str)
   Call Sleep(0.2)
   Str ="?D51f1f1f1f1f1f1f1f"
   Call PutStr_3(Str)
   Call Sleep(0.2)

   Str = "?c0"				' no cursor
   Call PutStr_3(Str)
   
   Str = "?y3?x00?l"			' cursor to beginning of line 3 and clear line
   Call PutStr_3(Str)

   For N = 0 to 25				' increasing bar
      Str = "?y3?x00"
      Call PutStr_3(Str)
      NN = 4 * N
      Num_5 = NN \ 5
      Remainder = NN Mod 5
      For J = 1 to Num_5
         Str = "?5"
         Call PutStr_3(Str)
      Next
      Str = "?" & CStr(Remainder)
      Call PutStr_3(Str)      
   Next

   For N = 0 to 25			' decreasing bar
      Str = "?y3?x00?l"
      Call PutStr_3(Str)
      NN = 100 - 4 * N
      Num_5 = NN \ 5
      Remainder = NN Mod 5
      For J = 1 to Num_5
         Str = "?5"
         Call PutStr_3(Str)
      Next
      Str = "?" & CStr(Remainder)
      Call PutStr_3(Str)      
   Next

   Do					' continually bring outputs 7 annd 6 high and low
      Str = "?H7?L6"
      Call PutStr_3(Str)
      Call Sleep(0.5)
      Str = "?L7?H6"
      Call PutStr_3(Str)
      Call Sleep(0.5)
   Loop
   
End Sub

